unit eepromWrite;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    addrEdit: TEdit;
    Label2: TLabel;
    dataEdit: TEdit;
    readBtn: TButton;
    writeBtn: TButton;
    Label3: TLabel;
    ReadLenEdit: TEdit;
    readDataEdit: TEdit;
    Label4: TLabel;
    StatusBar1: TStatusBar;
    Memo1: TMemo;
    Label5: TLabel;
    Label6: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure readBtnClick(Sender: TObject);
    procedure writeBtnClick(Sender: TObject);
    procedure dataEditChange(Sender: TObject);
    procedure OtherEditChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

uses
AIOUSB;

{$R *.DFM}

function StringReplaceC(const S: String; const OldChar, NewChar: Char): String;
var
  I: Integer;

begin
  Result := S;
  for I := 1 to Length(Result) do
    if Result[I] = OldChar then
      Result[I] := NewChar
  ;
end;

function StrToSpacedHex(Text: String): String;
const
  NybbleToHex: array[$0..$F] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F');
var
  I: Integer;

begin
  Result := StringOfChar(' ', Length(Text) * 3);
  for I := 1 to Length(Text) do begin
    Result[I * 3 - 2] := NybbleToHex[Byte(Text[I]) shr  4];
    Result[I * 3 - 1] := NybbleToHex[Byte(Text[I]) and $F];
  end;
end;

function ValidateHexText(S: String): String;
var
  Content: TStringList;
  Buf: String;
  I, L: Integer;

begin
  S := StringReplaceC(S, ',', #13);
  S := StringReplaceC(S, ' ', #13);
  Content := TStringList.Create;
  try
    Content.Text := S;
    S := '';
    for I := Content.Count - 1 downto 0 do //Clear blank bytes and double delimiters
      if Length(Content[I]) = 0 then
        Content.Delete(I)
    ;
    L := Content.Count;
    SetString(Result, nil, L);
    for I := 0 to L - 1 do begin
      Buf := Content[I];
      Result[I + 1] := Char(StrToInt('$' + Buf));    // can throw exception
    end;
  finally
    Content.Free;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
    Memo1.ScrollBars := ssVertical;
    if QueryDeviceInfo(diOnly, nil, nil, nil, nil, nil) <> ERROR_SUCCESS then begin
        MessageBox(0, 'This program requires exactly one ACCES USB device be attached to the computer',
                    'ERROR', MB_OK or MB_ICONEXCLAMATION
        );
        Application.ShowMainForm := False;
        Application.Terminate;
    end;
end;

procedure TForm1.readBtnClick(Sender: TObject);
var
  address : LongWord;
  Data: String;
  size : LongWord;
  Status: LongWord;

begin
    address := StrToIntDef('$' + addrEdit.Text, $200);

    if address > $1FF then begin
        StatusBar1.SimpleText := 'Error in address.';
        exit;
    end;

    size := StrToInt('$' + ReadLenEdit.Text);
    SetString(Data, nil, size);

    if Length(Data) + Integer(Address) > $200 then begin
      StatusBar1.SimpleText := 'Data goes past the end of the custom EEPROM.';
      Exit;
    end;

    Status := CustomEEPROMRead(diOnly, address, size, @Data[1]);
    if Status = ERROR_SUCCESS then
        StatusBar1.SimpleText := ''
    else begin
            StatusBar1.SimpleText := 'Error ' + IntToStr(Status) + ' reading data.';
            exit;
    end;

    readDataEdit.Text := StrToSpacedHex(Data);
end;

procedure TForm1.writeBtnClick(Sender: TObject);
var
  Address: LongWord;
  Data: String;
  Status: LongWord;

begin
    address := StrToIntDef('$' + addrEdit.Text, $200);

    if address > $1FF then
    begin
        StatusBar1.SimpleText := 'Error in address.';
        exit;
    end;

    Data := ValidateHexText(dataEdit.Text);

    Status := CustomEEPROMWrite(diOnly, address, Length(Data), @Data[1]);
    if Status = ERROR_SUCCESS then
        StatusBar1.SimpleText := 'Data written to EEPROM: ' + StrToSpacedHex(Data)
    else
        StatusBar1.SimpleText := 'Error ' + IntToStr(Status) + ' writing data.'
    ;
end;

procedure TForm1.dataEditChange(Sender: TObject);
var
  Data: String;
  Address: LongWord;

begin
  try
    address := StrToIntDef('$' + addrEdit.Text, $200);
    Data := ValidateHexText(dataEdit.Text);
    writeBtn.Caption := '&Write (' + IntToStr(Length(Data)) + 'h bytes)';
    StatusBar1.SimpleText := '';

    if Length(Data) + Integer(Address) > $200 then begin
      writeBtn.Caption := '&Write (too much!)';
      writeBtn.Enabled := False;
      StatusBar1.SimpleText := 'Data goes past the end of the custom EEPROM.';
      Exit;
    end;
    writeBtn.Enabled := True;
   StatusBar1.SimpleText := 'Data that will be written: ' + StrToSpacedHex(Data);
  except
    writeBtn.Caption := '&Write (bad data!)';
    writeBtn.Enabled := False;
    StatusBar1.SimpleText := 'Error in data.';
    Exit;
  end;


end;

procedure TForm1.OtherEditChange(Sender: TObject);
begin
  StatusBar1.SimpleText := '';
end;

end.
